@@ -13,7 +13,7 @@ use warnings;
 
 package Metabase::Archive::S3;
 BEGIN {
-  $Metabase::Archive::S3::VERSION = '0.012';
+  $Metabase::Archive::S3::VERSION = '0.013';
 }
 # ABSTRACT: Metabase storage using Amazon S3
 
@@ -24,6 +24,7 @@ use MooseX::Types::Path::Class;
 use Metabase::Fact;
 use Carp       ();
 use Data::GUID ();
+use Data::Stream::Bulk::Filter;
 use JSON 2 ();
 use Net::Amazon::S3;
 use Path::Class ();
@@ -128,15 +129,18 @@ sub extract {
     my ( $self, $guid ) = @_;
 
     my $s3_object = $self->s3_bucket->object( key => $self->prefix . lc $guid );
-    my $json = $s3_object->get;
+    return $self->_extract_struct( $s3_object );
+}
 
-    if ( $self->compressed ) {
-        $json = uncompress($json);
-    }
+sub _extract_struct {
+  my ( $self, $s3_object ) = @_;
 
-    my $object  = $self->_json->decode($json);
-
-    return $object;
+  my $json = $s3_object->get;
+  if ( $self->compressed ) {
+    $json = uncompress($json);
+  }
+  my $struct  = $self->_json->decode($json);
+  return $struct;
 }
 
 # DO NOT lc() GUID
@@ -147,6 +151,16 @@ sub delete {
     $s3_object->delete;
 }
 
+sub iterator {
+  my ($self) = @_;
+  return Data::Stream::Bulk::Filter->new(
+    stream => $self->s3_bucket->list( { prefix => $self->prefix } ),
+    filter => sub {
+      return [ map { $self->_extract_struct( $_ ) } @{ $_[0] } ];
+    },
+  );
+}
+
 1;
 
 
@@ -159,7 +173,7 @@ Metabase::Archive::S3 - Metabase storage using Amazon S3
 
 =head1 VERSION
 
-version 0.012
+version 0.013
 
 =head1 SYNOPSIS
 
@@ -176,7 +190,7 @@ version 0.012
 
 Store facts in Amazon S3.
 
-=for Pod::Coverage::TrustPod store extract delete
+=for Pod::Coverage::TrustPod store extract delete iterator
 
 =head1 USAGE
 
@@ -211,9 +225,21 @@ limitations under the License.
 
 =head1 AUTHORS
 
-  David Golden <dagolden@cpan.org>
-  Ricardo Signes <rjbs@cpan.org>
-  Leon Brocard <acme@cpan.org>
+=over 4
+
+=item *
+
+David Golden <dagolden@cpan.org>
+
+=item *
+
+Ricardo Signes <rjbs@cpan.org>
+
+=item *
+
+Leon Brocard <acme@cpan.org>
+
+=back
 
 =head1 COPYRIGHT AND LICENSE
 
